Module: EffectiveMembershipsOwner
- Extended by:
- ActiveSupport::Concern
- Defined in:
- app/models/concerns/effective_memberships_owner.rb
Overview
EffectiveMembershipsOwner
Mark your owner model with effective_memberships_owner to get all the includes
Defined Under Namespace
Modules: Base, ClassMethods
Instance Method Summary collapse
- #assign_member_role ⇒ Object
- #bad_standing_fees ⇒ Object
- #build_discount_fee(from:, date: nil) ⇒ Object
- #build_late_fee(category:, period:) ⇒ Object
- #build_membership_history(start_on: nil) ⇒ Object
-
#build_prorated_fee(date: nil) ⇒ Object
Instance Methods.
- #build_renewal_fee(category:, period:, late_on: nil, bad_standing_on: nil) ⇒ Object
-
#build_title_fee(title:, fee_type:, period:, price: nil, tax_exempt: nil, qb_item_name: nil, category: nil) ⇒ Object
Only thing optional is category, late_on and bad_standing_on.
- #max_fees_paid_period ⇒ Object
- #max_fees_paid_through_period ⇒ Object
- #membership_fees_paid? ⇒ Boolean
- #membership_history_continuous? ⇒ Boolean
-
#membership_history_errors ⇒ Object
Point out busted data.
- #membership_history_on(date) ⇒ Object
-
#membership_period_fee(category:, period:, except: nil) ⇒ Object
These should be singular fees anyway.
- #membership_removed? ⇒ Boolean
- #membership_removed_on ⇒ Object
- #outstanding_fee_payment_fees ⇒ Object
- #outstanding_fee_payment_orders ⇒ Object
- #registrar_action_categories(action) ⇒ Object
- #registrar_action_statuses(action) ⇒ Object
-
#update_member_role! ⇒ Object
This can be called by a script to recalculate the owner role based on current membership.
-
#update_membership_status! ⇒ Object
Called by the registrar.
Instance Method Details
#assign_member_role ⇒ Object
95 96 97 |
# File 'app/models/concerns/effective_memberships_owner.rb', line 95 def assign_member_role membership_present? ? add_role(:member) : remove_role(:member) end |
#bad_standing_fees ⇒ Object
117 118 119 |
# File 'app/models/concerns/effective_memberships_owner.rb', line 117 def bad_standing_fees fees.select { |fee| fee.bad_standing? } end |
#build_discount_fee(from:, date: nil) ⇒ Object
171 172 173 174 175 176 177 178 179 180 181 182 183 184 185 186 187 188 189 190 191 192 193 |
# File 'app/models/concerns/effective_memberships_owner.rb', line 171 def build_discount_fee(from:, date: nil) raise('must have an existing membership') unless membership.present? raise('existing membership category may not be same as from') if membership.category == from date ||= Time.zone.now price = from.discount_fee(date: date) period = EffectiveMemberships.Registrar.period(date: date) category = membership.category fee = fees.find { |fee| fee.fee_type == 'Discount' && fee.period == period && fee.category == category } || fees.build() return fee if fee.purchased? fee.assign_attributes( fee_type: 'Discount', category: category, period: period, price: price, tax_exempt: category.tax_exempt, qb_item_name: category.qb_item_name ) fee end |
#build_late_fee(category:, period:) ⇒ Object
258 259 260 261 262 263 264 265 266 267 268 269 270 271 272 273 274 275 276 277 278 279 280 281 282 |
# File 'app/models/concerns/effective_memberships_owner.rb', line 258 def build_late_fee(category:, period:) raise('must have an existing membership') unless membership.present? # Return existing but do not build yet fee = fees.find { |fee| fee.fee_type == 'Late' && fee.period == period && fee.category_id == category.id && fee.category_type == category.class.name } return fee if fee&.purchased? # Only continue if there is a late renewal fee for the same period renewal_fee = fees.find { |fee| fee.fee_type == 'Renewal' && fee.period == period && fee.category_id == category.id && fee.category_type == category.class.name } return unless fee.present? || renewal_fee&.late? # Build the late fee fee ||= fees.build() fee.assign_attributes( fee_type: 'Late', category: category, period: period, price: category.late_fee.to_i, tax_exempt: category.tax_exempt, qb_item_name: category.qb_item_name ) fee end |
#build_membership_history(start_on: nil) ⇒ Object
310 311 312 313 314 315 316 317 318 319 320 321 322 323 324 325 326 327 328 329 330 331 332 333 334 335 336 337 338 |
# File 'app/models/concerns/effective_memberships_owner.rb', line 310 def build_membership_history(start_on: nil) raise('expected membership to be present') unless membership.present? # The date of change start_on ||= Time.zone.now # End the other membership histories membership_histories.each { |history| history.end_on ||= start_on } # Snapshot of the current membership at this time history = membership_histories.build( start_on: start_on, end_on: nil, removed: membership.marked_for_destruction?, bad_standing: membership.bad_standing? ) unless history.removed? history.assign_attributes( number: membership.number, categories: membership.categories.map(&:to_s), category_ids: membership.categories.map(&:id), statuses: membership.statuses.map(&:to_s), status_ids: membership.statuses.map(&:id) ) end history end |
#build_prorated_fee(date: nil) ⇒ Object
Instance Methods
148 149 150 151 152 153 154 155 156 157 158 159 160 161 162 163 164 165 166 167 168 169 |
# File 'app/models/concerns/effective_memberships_owner.rb', line 148 def build_prorated_fee(date: nil) raise('must have an existing membership') unless membership.present? date ||= Time.zone.now price = membership.category.prorated_fee(date: date) period = EffectiveMemberships.Registrar.period(date: date) category = membership.category fee = fees.find { |fee| fee.fee_type == 'Prorated' && fee.period == period && fee.category == category } || fees.build() return fee if fee.purchased? fee.assign_attributes( fee_type: 'Prorated', category: category, period: period, price: price, tax_exempt: category.tax_exempt, qb_item_name: category.qb_item_name ) fee end |
#build_renewal_fee(category:, period:, late_on: nil, bad_standing_on: nil) ⇒ Object
226 227 228 229 230 231 232 233 234 235 236 237 238 239 240 241 242 243 244 245 246 247 248 249 250 251 252 253 254 255 256 |
# File 'app/models/concerns/effective_memberships_owner.rb', line 226 def build_renewal_fee(category:, period:, late_on: nil, bad_standing_on: nil) raise('must have an existing membership') unless membership.present? # Sanity check. # If there's already a purchased or unpurchased Prorated (or other membership period advancing fee) in this period # We shouldn't be building renewal fees for the same period a prorated fee is purchased in prorated = membership_period_fee(category: category, period: period, except: 'Renewal') raise('must not have an existing membership_period (prorated) fee in this period') if prorated.present? fee = fees.find { |fee| fee.fee_type == 'Renewal' && fee.period == period && fee.category_id == category.id && fee.category_type == category.class.name } return fee if fee&.purchased? # Build the renewal fee fee ||= fees.build() late_on ||= EffectiveMemberships.Registrar.late_fee_date(period: period) if category.create_late_fees? bad_standing_on ||= EffectiveMemberships.Registrar.bad_standing_date(period: period) if category.create_bad_standing? fee.assign_attributes( fee_type: 'Renewal', category: category, period: period, price: category.renewal_fee.to_i, tax_exempt: category.tax_exempt, qb_item_name: category.qb_item_name, late_on: late_on, bad_standing_on: bad_standing_on, ) fee end |
#build_title_fee(title:, fee_type:, period:, price: nil, tax_exempt: nil, qb_item_name: nil, category: nil) ⇒ Object
Only thing optional is category, late_on and bad_standing_on
196 197 198 199 200 201 202 203 204 205 206 207 208 209 210 211 212 213 214 215 216 217 218 |
# File 'app/models/concerns/effective_memberships_owner.rb', line 196 def build_title_fee(title:, fee_type:, period:, price: nil, tax_exempt: nil, qb_item_name: nil, category: nil) fee = fees.find do |fee| fee.fee_type == fee_type && fee.period == period && fee.title == title && (category.blank? || fee.category_id == category.id && fee.category_type == category.class.name) end return fee if fee&.purchased? # Build the title fee fee ||= fees.build() fee.assign_attributes( title: title, fee_type: fee_type, category: category, period: period, price: price, tax_exempt: tax_exempt, qb_item_name: qb_item_name ) fee end |
#max_fees_paid_period ⇒ Object
121 122 123 |
# File 'app/models/concerns/effective_memberships_owner.rb', line 121 def max_fees_paid_period fees.select { |fee| fee.membership_period_fee? && fee.purchased? }.map(&:period).max end |
#max_fees_paid_through_period ⇒ Object
125 126 127 128 |
# File 'app/models/concerns/effective_memberships_owner.rb', line 125 def max_fees_paid_through_period return nil if max_fees_paid_period.blank? EffectiveMemberships.Registrar.period_end_on(date: max_fees_paid_period) end |
#membership_fees_paid? ⇒ Boolean
105 106 107 |
# File 'app/models/concerns/effective_memberships_owner.rb', line 105 def membership_fees_paid? outstanding_fee_payment_fees.blank? && membership && membership.fees_paid? end |
#membership_history_continuous? ⇒ Boolean
382 383 384 385 386 |
# File 'app/models/concerns/effective_memberships_owner.rb', line 382 def membership_history_continuous? membership_histories.all? do |history| history.end_on.blank? || (history.end_on.present? && history.removed?) || membership_histories.find { |h| h.start_on == history.end_on }.present? end end |
#membership_history_errors ⇒ Object
Point out busted data
346 347 348 349 350 351 352 353 354 355 356 357 358 359 360 361 362 363 364 365 366 367 368 369 370 371 372 373 374 375 376 377 378 379 380 |
# File 'app/models/concerns/effective_memberships_owner.rb', line 346 def membership_history_errors return unless membership.present? return unless membership_histories.present? errors = [] history = membership_histories.first last_history = membership_histories.last # Check membership joined on date matches first history start date if membership.joined_on != history.start_on errors << "The joined date #{membership.joined_on.strftime('%F')} does not match the first history start date of #{history.start_on.strftime('%F')}. Please change the first history start date to #{membership.joined_on.strftime('%F')} or update the joined date above." end # Check that there is a membership history row if the registered date is unique if membership.joined_on != membership.registration_on && membership_histories.none? { |mh| mh.start_on == membership.registration_on } errors << "The registered date #{membership.registration_on.strftime('%F')} is missing a history with this date. Please create a history with a start date of #{membership.registration_on.strftime('%F')} or update the registered date above." end # Check numbers if membership.number.present? && membership_histories.none? { |history| history.number == membership.number } errors << "The membership number ##{membership.number} is missing a history with this number. Please create a history with the #{membership.number} number or update the membership number above." end # Check that the last history does not have an end_on date if last_history.end_on.present? && !last_history.removed? errors << "The most recent history must have a blank end date. Please remove the end date of the most recent history entry or create another history." elsif membership_histories.any? { |history| history.end_on.blank? && history != last_history } errors << "The end date must be present for all past histories. Please add the end date to all histories, except the most recent history." elsif !membership_history_continuous? errors << "The start and end dates are overlapping or non-continuous. Please make sure each history start date has a matching history end date" end errors end |
#membership_history_on(date) ⇒ Object
340 341 342 343 |
# File 'app/models/concerns/effective_memberships_owner.rb', line 340 def membership_history_on(date) raise('expected a date') unless date.respond_to?(:strftime) membership_histories.find { |history| (history.start_on..history.end_on).cover?(date) } # Ruby 2.6 supports endless ranges end |
#membership_period_fee(category:, period:, except: nil) ⇒ Object
These should be singular fees anyway.
221 222 223 224 |
# File 'app/models/concerns/effective_memberships_owner.rb', line 221 def membership_period_fee(category:, period:, except: nil) raise('expected except to be a string like Renewal') if except.present? && !except.kind_of?(String) fees.find { |fee| fee.membership_period_fee? && fee.period == period && fee.category_id == category.id && fee.category_type == category.class.name && (except.blank? || fee.fee_type != except) } end |
#membership_removed? ⇒ Boolean
130 131 132 |
# File 'app/models/concerns/effective_memberships_owner.rb', line 130 def membership_removed? membership.blank? && membership_histories.any? { |history| history.removed? } end |
#membership_removed_on ⇒ Object
134 135 136 137 |
# File 'app/models/concerns/effective_memberships_owner.rb', line 134 def membership_removed_on return nil unless membership_removed? membership_histories.find { |history| history.removed? }.start_on end |
#outstanding_fee_payment_fees ⇒ Object
109 110 111 |
# File 'app/models/concerns/effective_memberships_owner.rb', line 109 def outstanding_fee_payment_fees fees.select { |fee| fee.fee_payment_fee? && !fee.purchased? } end |
#outstanding_fee_payment_orders ⇒ Object
113 114 115 |
# File 'app/models/concerns/effective_memberships_owner.rb', line 113 def outstanding_fee_payment_orders orders.select { |order| order.parent_type.to_s.include?('FeePayment') && !order.purchased? } end |
#registrar_action_categories(action) ⇒ Object
139 140 141 |
# File 'app/models/concerns/effective_memberships_owner.rb', line 139 def registrar_action_categories(action) EffectiveMemberships.Category.sorted.all end |
#registrar_action_statuses(action) ⇒ Object
143 144 145 |
# File 'app/models/concerns/effective_memberships_owner.rb', line 143 def registrar_action_statuses(action) EffectiveMemberships.Status.sorted.all end |
#update_member_role! ⇒ Object
This can be called by a script to recalculate the owner role based on current membership
100 101 102 103 |
# File 'app/models/concerns/effective_memberships_owner.rb', line 100 def update_member_role! assign_member_role save! end |
#update_membership_status! ⇒ Object
Called by the registrar.
285 286 287 288 289 290 291 292 293 294 295 296 297 298 299 300 301 302 303 304 305 306 307 308 |
# File 'app/models/concerns/effective_memberships_owner.rb', line 285 def update_membership_status! raise('expected membership to be present') unless membership.present? # Assign fees paid through period membership.fees_paid_period = max_fees_paid_period() membership.fees_paid_through_period = max_fees_paid_through_period() # Assign in bad standing if membership.bad_standing_admin? # Nothing to do elsif bad_standing_fees.present? membership.bad_standing = true membership.bad_standing_reason = 'Unpaid Fees' else membership.bad_standing = false membership.bad_standing_reason = nil end if membership.bad_standing_changed? || membership_histories.blank? build_membership_history() end save! end |